# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import fixtures
from oslo_config import cfg

from tempest.common import credentials_factory as credentials
from tempest import config
from tempest.lib.common import dynamic_creds
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.identity.v2 import identity_client as v2_iden_client
from tempest.lib.services.identity.v2 import roles_client as v2_roles_client
from tempest.lib.services.identity.v2 import tenants_client as \
    v2_tenants_client
from tempest.lib.services.identity.v2 import token_client as v2_token_client
from tempest.lib.services.identity.v2 import users_client as v2_users_client
from tempest.lib.services.identity.v3 import domains_client
from tempest.lib.services.identity.v3 import identity_client as v3_iden_client
from tempest.lib.services.identity.v3 import projects_client as \
    v3_projects_client
from tempest.lib.services.identity.v3 import roles_client as v3_roles_client
from tempest.lib.services.identity.v3 import token_client as v3_token_client
from tempest.lib.services.identity.v3 import users_client as \
    v3_users_client
from tempest.lib.services.network import routers_client
from tempest.tests import base
from tempest.tests import fake_config
from tempest.tests.lib import fake_http
from tempest.tests.lib import fake_identity
from tempest.tests.lib.services import registry_fixture


class TestDynamicCredentialProvider(base.TestCase):

    fixed_params = {'name': 'test class',
                    'identity_version': 'v2',
                    'admin_role': 'admin',
                    'identity_uri': 'fake_uri'}

    token_client = v2_token_client
    iden_client = v2_iden_client
    roles_client = v2_roles_client
    tenants_client = v2_tenants_client
    users_client = v2_users_client
    token_client_class = token_client.TokenClient
    fake_response = fake_identity._fake_v2_response
    tenants_client_class = tenants_client.TenantsClient
    delete_tenant = 'delete_tenant'

    def setUp(self):
        super(TestDynamicCredentialProvider, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.useFixture(registry_fixture.RegistryFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)
        self.patchobject(self.token_client_class, 'raw_request',
                         self.fake_response)
        cfg.CONF.set_default('operator_role', 'FakeRole',
                             group='object-storage')
        self._mock_list_ec2_credentials('fake_user_id', 'fake_tenant_id')
        self.fixed_params.update(
            admin_creds=self._get_fake_admin_creds())

    def test_tempest_client(self):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self.assertIsInstance(creds.identity_admin_client,
                              self.iden_client.IdentityClient)

    def _get_fake_admin_creds(self):
        return credentials.get_credentials(
            fill_in=False,
            identity_version=self.fixed_params['identity_version'],
            username='fake_username', password='fake_password',
            tenant_name='fake_tenant')

    def _mock_user_create(self, id, name):
        user_fix = self.useFixture(fixtures.MockPatchObject(
            self.users_client.UsersClient,
            'create_user',
            return_value=(rest_client.ResponseBody
                          (200, {'user': {'id': id, 'name': name}}))))
        return user_fix

    def _mock_tenant_create(self, id, name):
        tenant_fix = self.useFixture(fixtures.MockPatchObject(
            self.tenants_client.TenantsClient,
            'create_tenant',
            return_value=(rest_client.ResponseBody
                          (200, {'tenant': {'id': id, 'name': name}}))))
        return tenant_fix

    def _mock_list_roles(self, id, name):
        roles_fix = self.useFixture(fixtures.MockPatchObject(
            self.roles_client.RolesClient,
            'list_roles',
            return_value=(rest_client.ResponseBody
                          (200,
                           {'roles': [{'id': id, 'name': name},
                                      {'id': '1', 'name': 'FakeRole'},
                                      {'id': '2', 'name': 'member'}]}))))
        return roles_fix

    def _mock_list_2_roles(self):
        roles_fix = self.useFixture(fixtures.MockPatchObject(
            self.roles_client.RolesClient,
            'list_roles',
            return_value=(rest_client.ResponseBody
                          (200,
                           {'roles': [{'id': '1234', 'name': 'role1'},
                                      {'id': '1', 'name': 'FakeRole'},
                                      {'id': '12345', 'name': 'role2'}]}))))
        return roles_fix

    def _mock_assign_user_role(self):
        tenant_fix = self.useFixture(fixtures.MockPatchObject(
            self.roles_client.RolesClient,
            'create_user_role_on_project',
            return_value=(rest_client.ResponseBody
                          (200, {}))))
        return tenant_fix

    def _mock_list_role(self):
        roles_fix = self.useFixture(fixtures.MockPatchObject(
            self.roles_client.RolesClient,
            'list_roles',
            return_value=(rest_client.ResponseBody
                          (200, {'roles': [
                              {'id': '1', 'name': 'FakeRole'},
                              {'id': '2', 'name': 'member'}]}))))
        return roles_fix

    def _mock_list_ec2_credentials(self, user_id, tenant_id):
        ec2_creds_fix = self.useFixture(fixtures.MockPatchObject(
            self.users_client.UsersClient,
            'list_user_ec2_credentials',
            return_value=(rest_client.ResponseBody
                          (200, {'credentials': [{
                                 'access': 'fake_access',
                                 'secret': 'fake_secret',
                                 'tenant_id': tenant_id,
                                 'user_id': user_id,
                                 'trust_id': None}]}))))
        return ec2_creds_fix

    def _mock_network_create(self, iso_creds, id, name):
        net_fix = self.useFixture(fixtures.MockPatchObject(
            iso_creds.networks_admin_client,
            'create_network',
            return_value={'network': {'id': id, 'name': name}}))
        return net_fix

    def _mock_subnet_create(self, iso_creds, id, name):
        subnet_fix = self.useFixture(fixtures.MockPatchObject(
            iso_creds.subnets_admin_client,
            'create_subnet',
            return_value={'subnet': {'id': id, 'name': name}}))
        return subnet_fix

    def _mock_router_create(self, id, name):
        router_fix = self.useFixture(fixtures.MockPatchObject(
            routers_client.RoutersClient,
            'create_router',
            return_value={'router': {'id': id, 'name': name}}))
        return router_fix

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_primary_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self._mock_user_create('1234', 'fake_prim_user')
        primary_creds = creds.get_primary_creds()
        self.assertEqual(primary_creds.username, 'fake_prim_user')
        self.assertEqual(primary_creds.tenant_name, 'fake_prim_tenant')
        # Verify IDs
        self.assertEqual(primary_creds.tenant_id, '1234')
        self.assertEqual(primary_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_admin_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_roles('1234', 'admin')
        self._mock_user_create('1234', 'fake_admin_user')
        self._mock_tenant_create('1234', 'fake_admin_tenant')

        user_mock = mock.patch.object(self.roles_client.RolesClient,
                                      'create_user_role_on_project')
        user_mock.start()
        self.addCleanup(user_mock.stop)
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            admin_creds = creds.get_admin_creds()
        user_mock.assert_has_calls([
            mock.call('1234', '1234', '1234')])
        self.assertEqual(admin_creds.username, 'fake_admin_user')
        self.assertEqual(admin_creds.tenant_name, 'fake_admin_tenant')
        # Verify IDs
        self.assertEqual(admin_creds.tenant_id, '1234')
        self.assertEqual(admin_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_project_alt_admin_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_roles('1234', 'admin')
        self._mock_user_create('1234', 'fake_alt_admin_user')
        self._mock_tenant_create('1234', 'fake_alt_admin')

        user_mock = mock.patch.object(self.roles_client.RolesClient,
                                      'create_user_role_on_project')
        user_mock.start()
        self.addCleanup(user_mock.stop)
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            alt_admin_creds = creds.get_project_alt_admin_creds()
        user_mock.assert_has_calls([
            mock.call('1234', '1234', '1234')])
        self.assertEqual(alt_admin_creds.username, 'fake_alt_admin_user')
        self.assertEqual(alt_admin_creds.project_name, 'fake_alt_admin')
        # Verify IDs
        self.assertEqual(alt_admin_creds.project_id, '1234')
        self.assertEqual(alt_admin_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_project_alt_member_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_tenant_create('1234', 'fake_alt_member')
        self._mock_user_create('1234', 'fake_alt_user')
        alt_member_creds = creds.get_project_alt_member_creds()
        self.assertEqual(alt_member_creds.username, 'fake_alt_user')
        self.assertEqual(alt_member_creds.project_name, 'fake_alt_member')
        # Verify IDs
        self.assertEqual(alt_member_creds.project_id, '1234')
        self.assertEqual(alt_member_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_project_alt_reader_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_roles('1234', 'reader')
        self._mock_tenant_create('1234', 'fake_alt_reader')
        self._mock_user_create('1234', 'fake_alt_user')
        alt_reader_creds = creds.get_project_alt_reader_creds()
        self.assertEqual(alt_reader_creds.username, 'fake_alt_user')
        self.assertEqual(alt_reader_creds.project_name, 'fake_alt_reader')
        # Verify IDs
        self.assertEqual(alt_reader_creds.project_id, '1234')
        self.assertEqual(alt_reader_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_role_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        self._mock_tenant_create('1234', 'fake_role_tenant')

        user_mock = mock.patch.object(self.roles_client.RolesClient,
                                      'create_user_role_on_project')
        user_mock.start()
        self.addCleanup(user_mock.stop)
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'])
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        args = map(lambda x: x[1], calls)
        args = list(args)
        self.assertIn(('1234', '1234', '1234'), args)
        self.assertIn(('1234', '1234', '12345'), args)
        self.assertEqual(role_creds.username, 'fake_role_user')
        self.assertEqual(role_creds.tenant_name, 'fake_role_tenant')
        # Verify IDs
        self.assertEqual(role_creds.tenant_id, '1234')
        self.assertEqual(role_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_role_creds_with_project_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        self._mock_tenant_create('1234', 'fake_role_project')

        user_mock = mock.patch.object(self.roles_client.RolesClient,
                                      'create_user_role_on_project')
        user_mock.start()
        self.addCleanup(user_mock.stop)
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='project')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        args = map(lambda x: x[1], calls)
        args = list(args)
        self.assertIn(('1234', '1234', '1234'), args)
        self.assertIn(('1234', '1234', '12345'), args)
        self.assertEqual(role_creds.username, 'fake_role_user')
        self.assertEqual(role_creds.project_name, 'fake_role_project')
        # Verify IDs
        self.assertEqual(role_creds.project_id, '1234')
        self.assertEqual(role_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def _test_get_same_role_creds_with_project_scope(self, MockRestClient,
                                                     scope=None):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        self._mock_tenant_create('1234', 'fake_role_project')
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope=scope)
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)

        # Fetch the same creds again
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope=scope)
        calls = user_mock1.mock_calls
        # Assert that previously created creds are return and no call to
        # role creation.
        self.assertEqual(len(calls), 0)
        # Check if previously created creds are returned.
        self.assertEqual(role_creds, role_creds_new)

    def test_get_same_role_creds_with_project_scope(self):
        self._test_get_same_role_creds_with_project_scope(scope='project')

    def test_get_same_role_creds_with_default_scope(self):
        self._test_get_same_role_creds_with_project_scope()

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def _test_get_different_role_creds_with_project_scope(
            self, MockRestClient, scope=None):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        self._mock_tenant_create('1234', 'fake_role_project')
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope=scope)
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        # Fetch the creds with one role different
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1'], scope=scope)
        calls = user_mock1.mock_calls
        # Because one role is different, assert that the role creation
        # is called with the 1 specified roles
        self.assertEqual(len(calls), 1)
        # Check new creds is created for new roles.
        self.assertNotEqual(role_creds, role_creds_new)

    def test_get_different_role_creds_with_project_scope(self):
        self._test_get_different_role_creds_with_project_scope(
            scope='project')

    def test_get_different_role_creds_with_default_scope(self):
        self._test_get_different_role_creds_with_project_scope()

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_all_cred_cleanup(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self._mock_user_create('1234', 'fake_prim_user')
        creds.get_primary_creds()
        self._mock_tenant_create('12345', 'fake_alt_tenant')
        self._mock_user_create('12345', 'fake_alt_user')
        creds.get_alt_creds()
        self._mock_tenant_create('123456', 'fake_admin_tenant')
        self._mock_user_create('123456', 'fake_admin_user')
        self._mock_list_roles('123456', 'admin')
        creds.get_admin_creds()
        user_mock = self.patchobject(self.users_client.UsersClient,
                                     'delete_user')
        tenant_mock = self.patchobject(self.tenants_client_class,
                                       self.delete_tenant)
        creds.clear_creds()
        # Verify user delete calls
        calls = user_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('1234', args)
        self.assertIn('12345', args)
        self.assertIn('123456', args)
        # Verify tenant delete calls
        calls = tenant_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('1234', args)
        self.assertIn('12345', args)
        self.assertIn('123456', args)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_alt_creds(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_alt_user')
        self._mock_tenant_create('1234', 'fake_alt_tenant')
        alt_creds = creds.get_alt_creds()
        self.assertEqual(alt_creds.username, 'fake_alt_user')
        self.assertEqual(alt_creds.tenant_name, 'fake_alt_tenant')
        # Verify IDs
        self.assertEqual(alt_creds.tenant_id, '1234')
        self.assertEqual(alt_creds.user_id, '1234')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_no_network_creation_with_config_set(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True, create_networks=False,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        net = mock.patch.object(creds.networks_admin_client,
                                'delete_network')
        net_mock = net.start()
        subnet = mock.patch.object(creds.subnets_admin_client,
                                   'delete_subnet')
        subnet_mock = subnet.start()
        router = mock.patch.object(creds.routers_admin_client,
                                   'delete_router')
        router_mock = router.start()

        primary_creds = creds.get_primary_creds()
        self.assertEqual(net_mock.mock_calls, [])
        self.assertEqual(subnet_mock.mock_calls, [])
        self.assertEqual(router_mock.mock_calls, [])
        network = primary_creds.network
        subnet = primary_creds.subnet
        router = primary_creds.router
        self.assertIsNone(network)
        self.assertIsNone(subnet)
        self.assertIsNone(router)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_network_creation(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self._mock_network_create(creds, '1234', 'fake_net')
        self._mock_subnet_create(creds, '1234', 'fake_subnet')
        self._mock_router_create('1234', 'fake_router')
        router_interface_mock = self.patch(
            'tempest.lib.services.network.routers_client.RoutersClient.'
            'add_router_interface')
        primary_creds = creds.get_primary_creds()
        router_interface_mock.assert_called_once_with('1234', subnet_id='1234')
        network = primary_creds.network
        subnet = primary_creds.subnet
        router = primary_creds.router
        self.assertEqual(network['id'], '1234')
        self.assertEqual(network['name'], 'fake_net')
        self.assertEqual(subnet['id'], '1234')
        self.assertEqual(subnet['name'], 'fake_subnet')
        self.assertEqual(router['id'], '1234')
        self.assertEqual(router['name'], 'fake_router')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_network_cleanup(self, MockRestClient):
        def side_effect(**args):
            return {"security_groups": [{"tenant_id": args['tenant_id'],
                                         "name": args['name'],
                                         "description": args['name'],
                                         "security_group_rules": [],
                                         "id": "sg-%s" % args['tenant_id']}]}
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            **self.fixed_params)
        # Create primary tenant and network
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self._mock_network_create(creds, '1234', 'fake_net')
        self._mock_subnet_create(creds, '1234', 'fake_subnet')
        self._mock_router_create('1234', 'fake_router')
        router_interface_mock = self.patch(
            'tempest.lib.services.network.routers_client.RoutersClient.'
            'add_router_interface')
        creds.get_primary_creds()
        router_interface_mock.assert_called_once_with('1234', subnet_id='1234')
        router_interface_mock.reset_mock()
        # Create alternate tenant and network
        self._mock_user_create('12345', 'fake_alt_user')
        self._mock_tenant_create('12345', 'fake_alt_tenant')
        self._mock_network_create(creds, '12345', 'fake_alt_net')
        self._mock_subnet_create(creds, '12345', 'fake_alt_subnet')
        self._mock_router_create('12345', 'fake_alt_router')
        creds.get_alt_creds()
        router_interface_mock.assert_called_once_with('12345',
                                                      subnet_id='12345')
        router_interface_mock.reset_mock()
        # Create admin tenant and networks
        self._mock_user_create('123456', 'fake_admin_user')
        self._mock_tenant_create('123456', 'fake_admin_tenant')
        self._mock_network_create(creds, '123456', 'fake_admin_net')
        self._mock_subnet_create(creds, '123456', 'fake_admin_subnet')
        self._mock_router_create('123456', 'fake_admin_router')
        self._mock_list_roles('123456', 'admin')
        creds.get_admin_creds()
        self.patchobject(self.users_client.UsersClient, 'delete_user')
        self.patchobject(self.tenants_client_class, self.delete_tenant)
        net = mock.patch.object(creds.networks_admin_client, 'delete_network')
        net_mock = net.start()
        subnet = mock.patch.object(creds.subnets_admin_client, 'delete_subnet')
        subnet_mock = subnet.start()
        router = mock.patch.object(creds.routers_admin_client, 'delete_router')
        router_mock = router.start()
        remove_router_interface_mock = self.patch(
            'tempest.lib.services.network.routers_client.RoutersClient.'
            'remove_router_interface')
        return_values = ({'status': 200}, {'ports': []})
        port_list_mock = mock.patch.object(creds.ports_admin_client,
                                           'list_ports',
                                           return_value=return_values)

        port_list_mock.start()
        secgroup_list_mock = mock.patch.object(
            creds.security_groups_admin_client,
            'list_security_groups',
            side_effect=side_effect)
        secgroup_list_mock.start()

        return_values = fake_http.fake_http_response({}, status=204), ''
        remove_secgroup_mock = self.patch(
            'tempest.lib.services.network.security_groups_client.'
            'SecurityGroupsClient.delete', return_value=return_values)
        creds.clear_creds()
        # Verify default security group delete
        calls = remove_secgroup_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('v2.0/security-groups/sg-1234', args)
        self.assertIn('v2.0/security-groups/sg-12345', args)
        self.assertIn('v2.0/security-groups/sg-123456', args)
        # Verify remove router interface calls
        calls = remove_router_interface_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: (x[1][0], x[2]), calls)
        args = list(args)
        self.assertIn(('1234', {'subnet_id': '1234'}), args)
        self.assertIn(('12345', {'subnet_id': '12345'}), args)
        self.assertIn(('123456', {'subnet_id': '123456'}), args)
        # Verify network delete calls
        calls = net_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('1234', args)
        self.assertIn('12345', args)
        self.assertIn('123456', args)
        # Verify subnet delete calls
        calls = subnet_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('1234', args)
        self.assertIn('12345', args)
        self.assertIn('123456', args)
        # Verify router delete calls
        calls = router_mock.mock_calls
        self.assertEqual(len(calls), 3)
        args = map(lambda x: x[1][0], calls)
        args = list(args)
        self.assertIn('1234', args)
        self.assertIn('12345', args)
        self.assertIn('123456', args)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_network_alt_creation(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_alt_user')
        self._mock_tenant_create('1234', 'fake_alt_tenant')
        self._mock_network_create(creds, '1234', 'fake_alt_net')
        self._mock_subnet_create(creds, '1234', 'fake_alt_subnet')
        self._mock_router_create('1234', 'fake_alt_router')
        router_interface_mock = self.patch(
            'tempest.lib.services.network.routers_client.RoutersClient.'
            'add_router_interface')
        alt_creds = creds.get_alt_creds()
        router_interface_mock.assert_called_once_with('1234', subnet_id='1234')
        network = alt_creds.network
        subnet = alt_creds.subnet
        router = alt_creds.router
        self.assertEqual(network['id'], '1234')
        self.assertEqual(network['name'], 'fake_alt_net')
        self.assertEqual(subnet['id'], '1234')
        self.assertEqual(subnet['name'], 'fake_alt_subnet')
        self.assertEqual(router['id'], '1234')
        self.assertEqual(router['name'], 'fake_alt_router')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_network_admin_creation(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_user_create('1234', 'fake_admin_user')
        self._mock_tenant_create('1234', 'fake_admin_tenant')
        self._mock_network_create(creds, '1234', 'fake_admin_net')
        self._mock_subnet_create(creds, '1234', 'fake_admin_subnet')
        self._mock_router_create('1234', 'fake_admin_router')
        router_interface_mock = self.patch(
            'tempest.lib.services.network.routers_client.RoutersClient.'
            'add_router_interface')
        self._mock_list_roles('123456', 'admin')
        admin_creds = creds.get_admin_creds()
        router_interface_mock.assert_called_once_with('1234', subnet_id='1234')
        network = admin_creds.network
        subnet = admin_creds.subnet
        router = admin_creds.router
        self.assertEqual(network['id'], '1234')
        self.assertEqual(network['name'], 'fake_admin_net')
        self.assertEqual(subnet['id'], '1234')
        self.assertEqual(subnet['name'], 'fake_admin_subnet')
        self.assertEqual(router['id'], '1234')
        self.assertEqual(router['name'], 'fake_admin_router')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_no_network_resources(self, MockRestClient):
        net_dict = {
            'network': False,
            'router': False,
            'subnet': False,
            'dhcp': False,
        }
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            network_resources=net_dict,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        net = mock.patch.object(creds.networks_admin_client,
                                'delete_network')
        net_mock = net.start()
        subnet = mock.patch.object(creds.subnets_admin_client,
                                   'delete_subnet')
        subnet_mock = subnet.start()
        router = mock.patch.object(creds.routers_admin_client,
                                   'delete_router')
        router_mock = router.start()

        primary_creds = creds.get_primary_creds()
        self.assertEqual(net_mock.mock_calls, [])
        self.assertEqual(subnet_mock.mock_calls, [])
        self.assertEqual(router_mock.mock_calls, [])
        network = primary_creds.network
        subnet = primary_creds.subnet
        router = primary_creds.router
        self.assertIsNone(network)
        self.assertIsNone(subnet)
        self.assertIsNone(router)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_router_without_network(self, MockRestClient):
        net_dict = {
            'network': False,
            'router': True,
            'subnet': False,
            'dhcp': False,
        }
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            network_resources=net_dict,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self.assertRaises(lib_exc.InvalidConfiguration,
                          creds.get_primary_creds)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_subnet_without_network(self, MockRestClient):
        net_dict = {
            'network': False,
            'router': False,
            'subnet': True,
            'dhcp': False,
        }
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            network_resources=net_dict,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self.assertRaises(lib_exc.InvalidConfiguration,
                          creds.get_primary_creds)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_dhcp_without_subnet(self, MockRestClient):
        net_dict = {
            'network': False,
            'router': False,
            'subnet': False,
            'dhcp': True,
        }
        creds = dynamic_creds.DynamicCredentialProvider(
            neutron_available=True,
            project_network_cidr='10.100.0.0/16', project_network_mask_bits=28,
            network_resources=net_dict,
            **self.fixed_params)
        self._mock_assign_user_role()
        self._mock_list_role()
        self._mock_user_create('1234', 'fake_prim_user')
        self._mock_tenant_create('1234', 'fake_prim_tenant')
        self.assertRaises(lib_exc.InvalidConfiguration,
                          creds.get_primary_creds)


class TestDynamicCredentialProviderV3(TestDynamicCredentialProvider):

    fixed_params = {'name': 'test class',
                    'identity_version': 'v3',
                    'admin_role': 'admin',
                    'identity_uri': 'fake_uri'}

    token_client = v3_token_client
    iden_client = v3_iden_client
    roles_client = v3_roles_client
    tenants_client = v3_projects_client
    users_client = v3_users_client
    token_client_class = token_client.V3TokenClient
    fake_response = fake_identity._fake_v3_response
    tenants_client_class = tenants_client.ProjectsClient
    delete_tenant = 'delete_project'

    def setUp(self):
        super(TestDynamicCredentialProviderV3, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.useFixture(fixtures.MockPatchObject(
            domains_client.DomainsClient, 'list_domains',
            return_value=dict(domains=[dict(id='default',
                                            name='Default')])))
        self.patchobject(self.roles_client.RolesClient,
                         'create_user_role_on_domain')

    def _mock_list_ec2_credentials(self, user_id, tenant_id):
        pass

    def _mock_tenant_create(self, id, name):
        project_fix = self.useFixture(fixtures.MockPatchObject(
            self.tenants_client.ProjectsClient,
            'create_project',
            return_value=(rest_client.ResponseBody
                          (200, {'project': {'id': id, 'name': name}}))))
        return project_fix

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_role_creds_with_system_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')

        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='system')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        args = map(lambda x: x[1], calls)
        args = list(args)
        self.assertIn(('1234', '1234'), args)
        self.assertIn(('1234', '12345'), args)
        self.assertEqual(role_creds.username, 'fake_role_user')
        self.assertEqual(role_creds.user_id, '1234')
        # Verify system scope
        self.assertEqual(role_creds.system, 'all')
        # Verify domain is default
        self.assertEqual(role_creds.domain_id, 'default')
        self.assertEqual(role_creds.domain_name, 'Default')

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_get_same_role_creds_with_system_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='system')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)

        # Fetch the same creds again
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='system')
        calls = user_mock1.mock_calls
        # Assert that previously created creds are return and no call to
        # role creation.
        self.assertEqual(len(calls), 0)
        # Verify system scope
        self.assertEqual(role_creds_new.system, 'all')
        # Check if previously created creds are returned.
        self.assertEqual(role_creds, role_creds_new)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_get_different_role_creds_with_system_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')

        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='system')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        # Verify system scope
        self.assertEqual(role_creds.system, 'all')
        # Fetch the creds with one role different
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1'], scope='system')
        calls = user_mock1.mock_calls
        # Because one role is different, assert that the role creation
        # is called with the 1 specified roles
        self.assertEqual(len(calls), 1)
        # Verify Scope
        self.assertEqual(role_creds_new.system, 'all')
        # Check new creds is created for new roles.
        self.assertNotEqual(role_creds, role_creds_new)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_role_creds_with_domain_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')

        domain = {
            "id": '12',
            "enabled": True,
            "name": "TestDomain"
        }

        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.cred_client.V3CredsClient.create_domain',
            return_value=domain))

        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_domain') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='domain')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        args = map(lambda x: x[1], calls)
        args = list(args)
        self.assertIn((domain['id'], '1234', '1234'), args)
        self.assertIn((domain['id'], '1234', '12345'), args)
        self.assertEqual(role_creds.username, 'fake_role_user')
        self.assertEqual(role_creds.user_id, '1234')
        # Verify creds are under new created domain
        self.assertEqual(role_creds.domain_id, domain['id'])
        self.assertEqual(role_creds.domain_name, domain['name'])
        # Verify that Scope is None
        self.assertIsNone(role_creds.system)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_get_same_role_creds_with_domain_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')

        domain = {
            "id": '12',
            "enabled": True,
            "name": "TestDomain"
        }

        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.cred_client.V3CredsClient.create_domain',
            return_value=domain))

        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_domain') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='domain')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        self.assertEqual(role_creds.user_id, '1234')
        # Verify Scope
        self.assertIsNone(role_creds.system)
        # Fetch the same creds again
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_domain') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='domain')
        calls = user_mock1.mock_calls
        # Assert that previously created creds are return and no call to
        # role creation.
        self.assertEqual(len(calls), 0)
        # Verify Scope
        self.assertIsNone(role_creds_new.system)
        # Check if previously created creds are returned.
        self.assertEqual(role_creds, role_creds_new)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_get_different_role_creds_with_domain_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')

        domain = {
            "id": '12',
            "enabled": True,
            "name": "TestDomain"
        }

        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.cred_client.V3CredsClient.create_domain',
            return_value=domain))

        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_domain') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='domain')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        self.assertEqual(role_creds.user_id, '1234')
        # Verify Scope
        self.assertIsNone(role_creds.system)
        # Fetch the same creds again
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_domain') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1'], scope='domain')
        calls = user_mock1.mock_calls
        # Because one role is different, assert that the role creation
        # is called with the 1 specified roles
        self.assertEqual(len(calls), 1)
        # Verify Scope
        self.assertIsNone(role_creds_new.system)
        # Check new creds is created for new roles.
        self.assertNotEqual(role_creds, role_creds_new)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_get_role_creds_with_different_scope(self, MockRestClient):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        self._mock_list_2_roles()
        self._mock_user_create('1234', 'fake_role_user')
        self._mock_tenant_create('1234', 'fake_role_project')
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_system') as user_mock:
            role_creds = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='system')
        calls = user_mock.mock_calls
        # Assert that the role creation is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        # Verify Scope
        self.assertEqual(role_creds.system, 'all')

        # Fetch the same role creds but with different scope
        with mock.patch.object(self.roles_client.RolesClient,
                               'create_user_role_on_project') as user_mock1:
            role_creds_new = creds.get_creds_by_roles(
                roles=['role1', 'role2'], scope='project')
        calls = user_mock1.mock_calls
        # Because scope is different, assert that the role creation
        # is called with the 2 specified roles
        self.assertEqual(len(calls), 2)
        # Verify Scope
        self.assertIsNone(role_creds_new.system)
        # Check that created creds are different
        self.assertNotEqual(role_creds, role_creds_new)

    @mock.patch('tempest.lib.common.rest_client.RestClient')
    def test_member_role_creation_with_duplicate(self, rest_client_mock):
        creds = dynamic_creds.DynamicCredentialProvider(**self.fixed_params)
        creds.creds_client = mock.MagicMock()
        creds.creds_client.create_user_role.side_effect = lib_exc.Conflict
        with mock.patch('tempest.lib.common.dynamic_creds.LOG') as log_mock:
            creds._create_creds()
            log_mock.warning.assert_called_once_with(
                "member role already exists, ignoring conflict.")
        creds.creds_client.assign_user_role.assert_called_once_with(
            mock.ANY, mock.ANY, 'member')



# Copyright 2013 NTT Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from oslo_utils.secretutils import md5
from tempest.api.object_storage import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

# Each segment, except for the final one, must be at least 1 megabyte
MIN_SEGMENT_SIZE = 1024 * 1024


class ObjectSloTest(base.BaseObjectTest):
    """Test static large object"""

    def setUp(self):
        super(ObjectSloTest, self).setUp()
        self.container_name = self.create_container()
        self.objects = []

    def tearDown(self):
        for obj in self.objects:
            test_utils.call_and_ignore_notfound_exc(
                self.object_client.delete_object,
                self.container_name, obj)
        self.container_client.delete_container(self.container_name)
        super(ObjectSloTest, self).tearDown()

    def _create_object(self, container_name, object_name, data, params=None):
        resp, _ = self.object_client.create_object(container_name,
                                                   object_name,
                                                   data,
                                                   params)
        self.objects.append(object_name)

        return resp

    def _create_manifest(self):
        # Create a manifest file for SLO uploading
        object_name = data_utils.rand_name(name='TestObject')
        object_name_base_1 = object_name + '_01'
        object_name_base_2 = object_name + '_02'
        data_size = MIN_SEGMENT_SIZE
        self.content = data_utils.random_bytes(data_size)
        self._create_object(self.container_name,
                            object_name_base_1,
                            self.content)
        self._create_object(self.container_name,
                            object_name_base_2,
                            self.content)

        path_object_1 = '/%s/%s' % (self.container_name,
                                    object_name_base_1)
        path_object_2 = '/%s/%s' % (self.container_name,
                                    object_name_base_2)
        data_manifest = [{'path': path_object_1,
                          'etag': md5(self.content,
                                      usedforsecurity=False).hexdigest(),
                          'size_bytes': data_size},
                         {'path': path_object_2,
                          'etag': md5(self.content,
                                      usedforsecurity=False).hexdigest(),
                          'size_bytes': data_size}]

        return json.dumps(data_manifest)

    def _create_large_object(self):
        # Create a large object for preparation of testing various SLO
        # features
        manifest = self._create_manifest()

        params = {'multipart-manifest': 'put'}
        object_name = data_utils.rand_name(name='TestObject')
        self._create_object(self.container_name,
                            object_name,
                            manifest,
                            params)
        return object_name

    def _assertHeadersSLO(self, resp, method):
        # When sending GET or HEAD requests to SLO the response contains
        # 'X-Static-Large-Object' header
        if method in ('GET', 'HEAD'):
            self.assertIn('x-static-large-object', resp)
            self.assertEqual(resp['x-static-large-object'], 'True')

        # Etag value of a large object is enclosed in double-quotations.
        # After etag quotes are checked they are removed and the response is
        # checked if all common headers are present and well formatted
        self.assertTrue(resp['etag'].startswith('\"'))
        self.assertTrue(resp['etag'].endswith('\"'))
        resp['etag'] = resp['etag'].strip('"')
        self.assertHeaders(resp, 'Object', method)

    @decorators.idempotent_id('2c3f24a6-36e8-4711-9aa2-800ee1fc7b5b')
    @utils.requires_ext(extension='slo', service='object')
    def test_upload_manifest(self):
        """Test creating static large object from multipart manifest"""
        manifest = self._create_manifest()

        params = {'multipart-manifest': 'put'}
        object_name = data_utils.rand_name(name='TestObject')
        resp = self._create_object(self.container_name,
                                   object_name,
                                   manifest,
                                   params)

        self._assertHeadersSLO(resp, 'PUT')

    @decorators.idempotent_id('e69ad766-e1aa-44a2-bdd2-bf62c09c1456')
    @utils.requires_ext(extension='slo', service='object')
    def test_list_large_object_metadata(self):
        """Test listing static large object metadata

        List static large object metadata using multipart manifest
        """
        object_name = self._create_large_object()

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)

        self._assertHeadersSLO(resp, 'HEAD')

    @decorators.idempotent_id('49bc49bc-dd1b-4c0f-904e-d9f10b830ee8')
    @utils.requires_ext(extension='slo', service='object')
    def test_retrieve_large_object(self):
        """Test listing static large object using multipart manifest"""
        object_name = self._create_large_object()

        resp, body = self.object_client.get_object(
            self.container_name,
            object_name)

        self._assertHeadersSLO(resp, 'GET')

        sum_data = self.content + self.content
        self.assertEqual(body, sum_data)

    @decorators.idempotent_id('87b6dfa1-abe9-404d-8bf0-6c3751e6aa77')
    @utils.requires_ext(extension='slo', service='object')
    def test_delete_large_object(self):
        """Test deleting static large object using multipart manifest"""
        object_name = self._create_large_object()

        params_del = {'multipart-manifest': 'delete'}
        resp, _ = self.object_client.delete_object(
            self.container_name,
            object_name,
            params=params_del)

        self.assertHeaders(resp, 'Object', 'DELETE')

        resp, _ = self.container_client.list_container_objects(
            self.container_name)
        self.assertEqual(int(resp['x-container-object-count']), 0)



# Copyright 2016 IBM Corp.
# Copyright 2017 AT&T Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_19 import servers as servers219

# The 2.26 microversion changes the server GET and (detailed) LIST responses to
# include the server 'tags' which is just a list of strings.

tag_items = {
    'type': 'array',
    'maxItems': 50,
    'items': {
        'type': 'string',
        'pattern': '^[^,/]*$',
        'maxLength': 60
    }
}

get_server = copy.deepcopy(servers219.get_server)
get_server['response_body']['properties']['server'][
    'properties'].update({'tags': tag_items})
get_server['response_body']['properties']['server'][
    'required'].append('tags')

list_servers_detail = copy.deepcopy(servers219.list_servers_detail)
list_servers_detail['response_body']['properties']['servers']['items'][
    'properties'].update({'tags': tag_items})
list_servers_detail['response_body']['properties']['servers']['items'][
    'required'].append('tags')

update_server = copy.deepcopy(servers219.update_server)
update_server['response_body']['properties']['server'][
    'properties'].update({'tags': tag_items})
update_server['response_body']['properties']['server'][
    'required'].append('tags')

rebuild_server = copy.deepcopy(servers219.rebuild_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'tags': tag_items})
rebuild_server['response_body']['properties']['server'][
    'required'].append('tags')

rebuild_server_with_admin_pass = copy.deepcopy(
    servers219.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'tags': tag_items})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('tags')

list_tags = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'tags': tag_items,
        },
        'additionalProperties': False,
        'required': ['tags']
    }
}

update_all_tags = copy.deepcopy(list_tags)

delete_all_tags = {'status_code': [204]}

check_tag_existence = {'status_code': [204]}

update_tag = {
    'status_code': [201, 204],
    'response_header': {
        'type': 'object',
        'properties': {
            'location': {
                'type': 'string'
            }
        },
        'required': ['location']
    }
}

delete_tag = {'status_code': [204]}

# NOTE(gmann): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.19 ******
list_servers = copy.deepcopy(servers219.list_servers)
show_server_diagnostics = copy.deepcopy(servers219.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers219.get_remote_consoles)
attach_volume = copy.deepcopy(servers219.attach_volume)
show_volume_attachment = copy.deepcopy(servers219.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers219.list_volume_attachments)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import hashlib
import hmac
import time
from urllib import parse as urlparse

from tempest.api.object_storage import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class ObjectTempUrlTest(base.BaseObjectTest):
    """Test object temp url"""

    @classmethod
    def resource_setup(cls):
        super(ObjectTempUrlTest, cls).resource_setup()
        # create a container
        cls.container_name = cls.create_container()

        # update account metadata
        cls.key = 'Meta'
        cls.metadatas = []
        metadata = {'Temp-URL-Key': cls.key}
        cls.metadatas.append(metadata)
        cls.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)

        # create an object
        cls.object_name, cls.content = cls.create_object(cls.container_name)

    @classmethod
    def resource_cleanup(cls):
        for metadata in cls.metadatas:
            cls.account_client.create_update_or_delete_account_metadata(
                delete_metadata=metadata)

        cls.delete_containers()

        super(ObjectTempUrlTest, cls).resource_cleanup()

    def setUp(self):
        super(ObjectTempUrlTest, self).setUp()

        # make sure the metadata has been set
        account_client_metadata, _ = \
            self.account_client.list_account_metadata()
        self.assertIn('x-account-meta-temp-url-key',
                      account_client_metadata)
        self.assertEqual(
            account_client_metadata['x-account-meta-temp-url-key'],
            self.key)

    def _get_expiry_date(self, expiration_time=1000):
        return int(time.time() + expiration_time)

    def _get_temp_url(self, container, object_name, method, expires,
                      key):
        """Create the temporary URL."""

        path = "%s/%s/%s" % (
            urlparse.urlparse(self.object_client.base_url).path,
            container, object_name)

        hmac_body = '%s\n%s\n%s' % (method, expires, path)
        sig = hmac.new(
            key.encode(), hmac_body.encode(), hashlib.sha1
        ).hexdigest()

        url = "%s/%s?temp_url_sig=%s&temp_url_expires=%s" % (container,
                                                             object_name,
                                                             sig, expires)

        return url

    @decorators.idempotent_id('f91c96d4-1230-4bba-8eb9-84476d18d991')
    @utils.requires_ext(extension='tempurl', service='object')
    def test_get_object_using_temp_url(self):
        """Test getting object using temp url"""
        expires = self._get_expiry_date()

        # get a temp URL for the created object
        url = self._get_temp_url(self.container_name,
                                 self.object_name, "GET",
                                 expires, self.key)

        # trying to get object using temp url within expiry time
        resp, body = self.object_client.get(url)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, self.content)

        # Testing a HEAD on this Temp URL
        resp, _ = self.object_client.head(url)
        self.assertHeaders(resp, 'Object', 'HEAD')

    @decorators.idempotent_id('671f9583-86bd-4128-a034-be282a68c5d8')
    @utils.requires_ext(extension='tempurl', service='object')
    def test_get_object_using_temp_url_key_2(self):
        """Test getting object using metadata 'Temp-URL-Key-2'"""
        key2 = 'Meta2-'
        metadata = {'Temp-URL-Key-2': key2}
        self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)
        self.metadatas.append(metadata)

        # make sure the metadata has been set
        account_client_metadata, _ = \
            self.account_client.list_account_metadata()
        self.assertIn('x-account-meta-temp-url-key-2',
                      account_client_metadata)
        self.assertEqual(
            account_client_metadata['x-account-meta-temp-url-key-2'],
            key2)

        expires = self._get_expiry_date()
        url = self._get_temp_url(self.container_name,
                                 self.object_name, "GET",
                                 expires, key2)
        _, body = self.object_client.get(url)
        self.assertEqual(body, self.content)

    @decorators.idempotent_id('9b08dade-3571-4152-8a4f-a4f2a873a735')
    @utils.requires_ext(extension='tempurl', service='object')
    def test_put_object_using_temp_url(self):
        """Test putting object using temp url"""
        new_data = data_utils.random_bytes(size=len(self.object_name))

        expires = self._get_expiry_date()
        url = self._get_temp_url(self.container_name,
                                 self.object_name, "PUT",
                                 expires, self.key)

        # trying to put random data in the object using temp url
        resp, _ = self.object_client.put(url, new_data, None)
        self.assertHeaders(resp, 'Object', 'PUT')

        # Testing a HEAD on this Temp URL
        resp, _ = self.object_client.head(url)
        self.assertHeaders(resp, 'Object', 'HEAD')

        # Validate that the content of the object has been modified
        url = self._get_temp_url(self.container_name,
                                 self.object_name, "GET",
                                 expires, self.key)

        _, body = self.object_client.get(url)
        self.assertEqual(body, new_data)

    @decorators.idempotent_id('249a0111-5ad3-4534-86a7-1993d55f9185')
    @utils.requires_ext(extension='tempurl', service='object')
    def test_head_object_using_temp_url(self):
        """Test HEAD operation of object using temp url"""
        expires = self._get_expiry_date()

        # get a temp URL for the created object
        url = self._get_temp_url(self.container_name,
                                 self.object_name, "HEAD",
                                 expires, self.key)

        # Testing a HEAD on this Temp URL
        resp, _ = self.object_client.head(url)
        self.assertHeaders(resp, 'Object', 'HEAD')

    @decorators.idempotent_id('9d9cfd90-708b-465d-802c-e4a8090b823d')
    @utils.requires_ext(extension='tempurl', service='object')
    def test_get_object_using_temp_url_with_inline_query_parameter(self):
        """Test getting object using temp url with inline query parameter"""
        expires = self._get_expiry_date()

        # get a temp URL for the created object
        url = self._get_temp_url(self.container_name, self.object_name, "GET",
                                 expires, self.key)
        url = url + '&inline'

        # trying to get object using temp url within expiry time
        resp, body = self.object_client.get(url)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, self.content)
        self.assertEqual(resp['content-disposition'], 'inline')



# Copyright 2015 OpenStack Foundation.
# Copyright 2015, Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib import decorators


class TestApiDiscovery(base.BaseIdentityV2Test):
    """Tests for identity v2 API discovery features."""

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('ea889a68-a15f-4166-bfb1-c12456eae853')
    def test_api_version_resources(self):
        """Test showing identity v2 api version resources"""
        descr = self.non_admin_client.show_api_description()['version']
        expected_resources = ('id', 'links', 'media-types', 'status',
                              'updated')

        keys = descr.keys()
        for res in expected_resources:
            self.assertIn(res, keys)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('007a0be0-78fe-4fdb-bbee-e9216cc17bb2')
    def test_api_media_types(self):
        """Test showing identity v2 api version media type"""
        descr = self.non_admin_client.show_api_description()['version']
        # Get MIME type bases and descriptions
        media_types = [(media_type['base'], media_type['type']) for
                       media_type in descr['media-types']]
        # These are supported for API version 2
        supported_types = [('application/json',
                            'application/vnd.openstack.identity-v2.0+json')]

        # Check if supported types exist in response body
        for s_type in supported_types:
            self.assertIn(s_type, media_types)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('77fd6be0-8801-48e6-b9bf-38cdd2f253ec')
    def test_api_version_statuses(self):
        """Test showing identity v2 api version status"""
        descr = self.non_admin_client.show_api_description()['version']
        status = descr['status'].lower()
        supported_statuses = ['current', 'stable', 'experimental',
                              'supported', 'deprecated']

        self.assertIn(status, supported_statuses)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import limits_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestLimitsClient(base.BaseServiceTest):

    def setUp(self):
        super(TestLimitsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = limits_client.LimitsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_show_limits(self, bytes_body=False):
        expected = {
            "limits": {
                "rate": [],
                "absolute": {
                    "maxServerMeta": 128,
                    "maxPersonality": 5,
                    "totalServerGroupsUsed": 0,
                    "maxImageMeta": 128,
                    "maxPersonalitySize": 10240,
                    "maxServerGroups": 10,
                    "maxSecurityGroupRules": 20,
                    "maxTotalKeypairs": 100,
                    "totalCoresUsed": 0,
                    "totalRAMUsed": 0,
                    "totalInstancesUsed": 0,
                    "maxSecurityGroups": 10,
                    "totalFloatingIpsUsed": 0,
                    "maxTotalCores": 20,
                    "totalSecurityGroupsUsed": 0,
                    "maxTotalFloatingIps": 10,
                    "maxTotalInstances": 10,
                    "maxTotalRAMSize": 51200,
                    "maxServerGroupMembers": 10
                    }
            }
        }

        self.check_service_client_function(
            self.client.show_limits,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body)

    def test_show_limits_with_str_body(self):
        self._test_show_limits()

    def test_show_limits_with_bytes_body(self):
        self._test_show_limits(bytes_body=True)



# Copyright 2015 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
from oslo_config import cfg

from tempest.common import credentials_factory as credentials
from tempest import config
from tempest.tests import base
from tempest.tests import fake_config


class TestAdminAvailable(base.TestCase):

    identity_version = 'v2'

    def setUp(self):
        super(TestAdminAvailable, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)

    def run_test(self, dynamic_creds, use_accounts_file, admin_creds):

        cfg.CONF.set_default('use_dynamic_credentials',
                             dynamic_creds, group='auth')
        if use_accounts_file:
            accounts = [{'username': 'u1',
                         'project_name': 't1',
                         'password': 'p'},
                        {'username': 'u2',
                         'project_name': 't2',
                         'password': 'p'}]
            if admin_creds == 'role':
                accounts.append({'username': 'admin',
                                 'project_name': 'admin',
                                 'password': 'p',
                                 'roles': ['admin']})
            elif admin_creds == 'type':
                accounts.append({'username': 'admin',
                                 'project_name': 'admin',
                                 'password': 'p',
                                 'types': ['admin']})
            self.useFixture(fixtures.MockPatch(
                'tempest.lib.common.preprov_creds.read_accounts_yaml',
                return_value=accounts))
            cfg.CONF.set_default('test_accounts_file',
                                 use_accounts_file, group='auth')
            self.useFixture(fixtures.MockPatch('os.path.isfile',
                                               return_value=True))
        else:
            self.useFixture(fixtures.MockPatch('os.path.isfile',
                                               return_value=False))
            if admin_creds:
                username = 'u'
                project = 't'
                password = 'p'
                domain = 'd'
            else:
                username = None
                project = None
                password = None
                domain = None

            cfg.CONF.set_default('admin_username', username, group='auth')
            cfg.CONF.set_default('admin_project_name', project, group='auth')
            cfg.CONF.set_default('admin_password', password, group='auth')
            cfg.CONF.set_default('admin_domain_name', domain, group='auth')

        expected = admin_creds is not None or dynamic_creds
        observed = credentials.is_admin_available(
            identity_version=self.identity_version)
        self.assertEqual(expected, observed)

    # Dynamic credentials implies admin so only one test case for True
    def test__dynamic_creds__accounts_file__no_admin(self):
        self.run_test(dynamic_creds=True,
                      use_accounts_file=True,
                      admin_creds=None)

    def test__no_dynamic_creds__accounts_file__no_admin(self):
        self.run_test(dynamic_creds=False,
                      use_accounts_file=True,
                      admin_creds=None)

    def test__no_dynamic_creds__accounts_file__admin_role(self):
        self.run_test(dynamic_creds=False,
                      use_accounts_file=True,
                      admin_creds='role')

    def test__no_dynamic_creds__accounts_file__admin_type(self):
        self.run_test(dynamic_creds=False,
                      use_accounts_file=True,
                      admin_creds='type')

    def test__no_dynamic_creds__no_accounts_file__no_admin(self):
        self.run_test(dynamic_creds=False,
                      use_accounts_file=False,
                      admin_creds=None)

    def test__no_dynamic_creds__no_accounts_file__admin(self):
        self.run_test(dynamic_creds=False,
                      use_accounts_file=False,
                      admin_creds='role')


class TestAdminAvailableV3(TestAdminAvailable):

    identity_version = 'v3'



# Copyright 2017 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ServerDiagnosticsNegativeTest(base.BaseV2ComputeAdminTest):
    """Negative tests of server diagnostics"""

    @classmethod
    def setup_clients(cls):
        super(ServerDiagnosticsNegativeTest, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e84e2234-60d2-42fa-8b30-e2d3049724ac')
    def test_get_server_diagnostics_by_non_admin(self):
        """Test getting server diagnostics by non-admin user is forbidden

        Non-admin user cannot view server diagnostics according to policy.
        """
        server_id = self.create_test_server(wait_until='ACTIVE')['id']
        self.assertRaises(lib_exc.Forbidden,
                          self.client.show_server_diagnostics, server_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class MetadataNamespaceObjectsTest(base.BaseV2ImageAdminTest):
    """Test the Metadata definition namespace objects basic functionality"""

    def _create_namespace_object(self, namespace):
        object_name = data_utils.rand_name(self.__class__.__name__ + '-object')
        namespace_object = self.namespace_objects_client.\
            create_namespace_object(namespace['namespace'], name=object_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_objects_client.delete_namespace_object,
                        namespace['namespace'], object_name)
        return namespace_object

    @decorators.idempotent_id('b1a3775e-3b5c-4f6a-a3b4-1ba3574ae718')
    def test_create_update_delete_meta_namespace_objects(self):
        """Test creating/updating/deleting image metadata namespace objects"""
        # Create a namespace
        namespace = self.create_namespace()
        # Create a namespace object
        body = self._create_namespace_object(namespace)
        # Update a namespace object
        up_object_name = data_utils.rand_name('update-object')
        body = self.namespace_objects_client.update_namespace_object(
            namespace['namespace'], body['name'],
            name=up_object_name)
        self.assertEqual(up_object_name, body['name'])
        # Delete a namespace object
        self.namespace_objects_client.delete_namespace_object(
            namespace['namespace'], up_object_name)
        # List namespace objects and validate deletion
        namespace_objects = [
            namespace_object['name'] for namespace_object in
            self.namespace_objects_client.list_namespace_objects(
                namespace['namespace'])['objects']]
        self.assertNotIn(up_object_name, namespace_objects)

    @decorators.idempotent_id('a2a3615e-3b5c-3f6a-a2b1-1ba3574ae738')
    def test_list_meta_namespace_objects(self):
        """Test listing image metadata namespace objects"""
        # Create a namespace object
        namespace = self.create_namespace()
        meta_namespace_object = self._create_namespace_object(namespace)
        # List namespace objects
        namespace_objects = [
            namespace_object['name'] for namespace_object in
            self.namespace_objects_client.list_namespace_objects(
                namespace['namespace'])['objects']]
        self.assertIn(meta_namespace_object['name'], namespace_objects)

    @decorators.idempotent_id('b1a3674e-3b4c-3f6a-a3b4-1ba3573ca768')
    def test_show_meta_namespace_objects(self):
        """Test showing image metadata namespace object"""
        # Create a namespace object
        namespace = self.create_namespace()
        namespace_object = self._create_namespace_object(namespace)
        # Show a namespace object
        body = self.namespace_objects_client.show_namespace_object(
            namespace['namespace'], namespace_object['name'])
        self.assertEqual(namespace_object['name'], body['name'])



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import groups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestGroupsClient(base.BaseServiceTest):
    FAKE_CREATE_GROUP = {
        'group': {
            'description': 'Tempest Group Description',
            'domain_id': 'TempestDomain',
            'name': 'Tempest Group',
        }
    }

    FAKE_GROUP_INFO = {
        'group': {
            'description': 'Tempest Group Description',
            'domain_id': 'TempestDomain',
            'id': '6e13e2068cf9466e98950595baf6bb35',
            'links': {
                'self': 'http://example.com/identity/v3/groups/' +
                        '6e13e2068cf9466e98950595baf6bb35'
            },
            'name': 'Tempest Group',
        }
    }

    FAKE_GROUP_LIST = {
        'links': {
            'self': 'http://example.com/identity/v3/groups',
            'previous': None,
            'next': None,
        },
        'groups': [
            {
                'description': 'Tempest Group One Description',
                'domain_id': 'TempestDomain',
                'id': '1c92f3453ed34291a074b87493455b8f',
                'links': {
                    'self': 'http://example.com/identity/v3/groups/' +
                            '1c92f3453ed34291a074b87493455b8f'
                },
                'name': 'Tempest Group One',
            },
            {
                'description': 'Tempest Group Two Description',
                'domain_id': 'TempestDomain',
                'id': 'ce9e7dafed3b4877a7d4466ed730a9ee',
                'links': {
                    'self': 'http://example.com/identity/v3/groups/' +
                            'ce9e7dafed3b4877a7d4466ed730a9ee'
                },
                'name': 'Tempest Group Two',
            },
        ]
    }

    FAKE_USER_LIST = {
        'links': {
            'self': 'http://example.com/identity/v3/groups/' +
                    '6e13e2068cf9466e98950595baf6bb35/users',
            'previous': None,
            'next': None,
        },
        'users': [
            {
                'domain_id': 'TempestDomain',
                'description': 'Tempest Test User One Description',
                'enabled': True,
                'id': '642688fa65a84217b86cef3c063de2b9',
                'name': 'TempestUserOne',
                'links': {
                    'self': 'http://example.com/identity/v3/users/' +
                            '642688fa65a84217b86cef3c063de2b9'
                }
            },
            {
                'domain_id': 'TempestDomain',
                'description': 'Tempest Test User Two Description',
                'enabled': True,
                'id': '1048ead6f8ef4a859b44ffbce3ac0b52',
                'name': 'TempestUserTwo',
                'links': {
                    'self': 'http://example.com/identity/v3/users/' +
                            '1048ead6f8ef4a859b44ffbce3ac0b52'
                }
            },
        ]
    }

    def setUp(self):
        super(TestGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = groups_client.GroupsClient(fake_auth, 'identity',
                                                 'regionOne')

    def _test_create_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP,
            bytes_body,
            status=201,
        )

    def _test_show_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_GROUP_INFO,
            bytes_body,
            group_id='6e13e2068cf9466e98950595baf6bb35',
        )

    def _test_list_groups(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_GROUP_LIST,
            bytes_body,
        )

    def _test_update_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_group,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_GROUP_INFO,
            bytes_body,
            group_id='6e13e2068cf9466e98950595baf6bb35',
            name='NewName',
        )

    def _test_list_users_in_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_group_users,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_USER_LIST,
            bytes_body,
            group_id='6e13e2068cf9466e98950595baf6bb35',
        )

    def test_create_group_with_string_body(self):
        self._test_create_group()

    def test_create_group_with_bytes_body(self):
        self._test_create_group(bytes_body=True)

    def test_show_group_with_string_body(self):
        self._test_show_group()

    def test_show_group_with_bytes_body(self):
        self._test_show_group(bytes_body=True)

    def test_list_groups_with_string_body(self):
        self._test_list_groups()

    def test_list_groups_with_bytes_body(self):
        self._test_list_groups(bytes_body=True)

    def test_update_group_with_string_body(self):
        self._test_update_group()

    def test_update_group_with_bytes_body(self):
        self._test_update_group(bytes_body=True)

    def test_list_users_in_group_with_string_body(self):
        self._test_list_users_in_group()

    def test_list_users_in_group_with_bytes_body(self):
        self._test_list_users_in_group(bytes_body=True)

    def test_delete_group(self):
        self.check_service_client_function(
            self.client.delete_group,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            group_id='6e13e2068cf9466e98950595baf6bb35',
            status=204,
        )

    def test_add_user_to_group(self):
        self.check_service_client_function(
            self.client.add_group_user,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            status=204,
            group_id='6e13e2068cf9466e98950595baf6bb35',
            user_id='642688fa65a84217b86cef3c063de2b9',
        )

    def test_check_user_in_group(self):
        self.check_service_client_function(
            self.client.check_group_user_existence,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            status=204,
            group_id='6e13e2068cf9466e98950595baf6bb35',
            user_id='642688fa65a84217b86cef3c063de2b9',
        )

    def test_delete_group_user(self):
        self.check_service_client_function(
            self.client.delete_group_user,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            group_id='6e13e2068cf9466e98950595baf6bb35',
            user_id='642688fa65a84217b86cef3c063de2b9',
        )



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF

LOG = logging.getLogger(__name__)


class TestServerAdvancedOps(manager.ScenarioTest):

    """The test suite for server advanced operations

    This test case stresses some advanced server instance operations:
     * Sequence suspend resume
    """

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(network=True, subnet=True)
        super(TestServerAdvancedOps, cls).setup_credentials()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('949da7d5-72c8-4808-8802-e3d70df98e2c')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @utils.services('compute')
    def test_server_sequence_suspend_resume(self):
        # We create an instance for use in this test
        instance_id = self.create_server()['id']

        for _ in range(2):
            LOG.debug("Suspending instance %s", instance_id)
            self.servers_client.suspend_server(instance_id)
            waiters.wait_for_server_status(self.servers_client, instance_id,
                                           'SUSPENDED')

            LOG.debug("Resuming instance %s", instance_id)
            self.servers_client.resume_server(instance_id)
            waiters.wait_for_server_status(self.servers_client, instance_id,
                                           'ACTIVE')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class IdentityClient(rest_client.RestClient):
    api_version = "v2.0"

    def show_api_description(self):
        """Retrieves info about the v2.0 Identity API"""
        url = ''
        resp, body = self.get(url)
        self.expected_success([200, 203], resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_token(self, token_id):
        """Get token details."""
        resp, body = self.get("tokens/%s" % token_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_token(self, token_id):
        """Delete a token."""
        resp, body = self.delete("tokens/%s" % token_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_extensions(self):
        """List all the extensions."""
        resp, body = self.get('/extensions')
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_endpoints_for_token(self, token_id):
        """List endpoints for a token """
        resp, body = self.get("tokens/%s/endpoints" % token_id)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_token_existence(self, token_id, **params):
        """Validates a token and confirms that it belongs to a tenant.

        For a full list of available parameters, please refer to the
        official API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/#validate-token
        """
        url = "tokens/%s" % token_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.head(url)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import interfaces as schema
from tempest.lib.api_schema.response.compute.v2_70 import interfaces as \
    schemav270
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class InterfacesClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.69', 'schema': schema},
        {'min': '2.70', 'max': None, 'schema': schemav270}]

    def list_interfaces(self, server_id):
        resp, body = self.get('servers/%s/os-interface' % server_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_interfaces, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_interface(self, server_id, **kwargs):
        """Create an interface.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-interface
        """
        post_body = {'interfaceAttachment': kwargs}
        post_body = json.dumps(post_body)
        resp, body = self.post('servers/%s/os-interface' % server_id,
                               body=post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_create_interfaces, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_interface(self, server_id, port_id):
        resp, body = self.get('servers/%s/os-interface/%s' % (server_id,
                                                              port_id))
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_create_interfaces, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_interface(self, server_id, port_id):
        resp, body = self.delete('servers/%s/os-interface/%s' % (server_id,
                                                                 port_id))
        self.validate_response(schema.delete_interface, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class MetadataNamespacePropertiesTest(base.BaseV2ImageAdminTest):
    """Test the Metadata definition namespace property basic functionality"""

    @decorators.idempotent_id('b1a3765e-3a5d-4f6d-a3a7-3ca3476ae768')
    def test_basic_meta_def_namespace_property(self):
        """Test operations of image metadata definition namespace property"""
        # Get the available resource types and use one resource_type
        body = self.resource_types_client.list_resource_types()
        resource_name = body['resource_types'][0]['name']
        enum = ["xen", "qemu", "kvm", "lxc", "uml", "vmware", "hyperv"]
        # Create a namespace
        namespace = self.create_namespace()
        # Create resource type association
        body = self.resource_types_client.create_resource_type_association(
            namespace['namespace'], name=resource_name)
        # Create a property
        property_title = data_utils.rand_name('property')
        body = self.namespace_properties_client.create_namespace_property(
            namespace=namespace['namespace'], title=property_title,
            name=resource_name, type="string", enum=enum)
        self.assertEqual(property_title, body['title'])
        # Show namespace property
        body = self.namespace_properties_client.show_namespace_properties(
            namespace['namespace'], resource_name)
        self.assertEqual(resource_name, body['name'])
        # Update namespace property
        update_property_title = data_utils.rand_name('update-property')
        body = self.namespace_properties_client.update_namespace_properties(
            namespace['namespace'], resource_name,
            title=update_property_title, type="string",
            enum=enum, name=resource_name)
        self.assertEqual(update_property_title, body['title'])
        # Delete namespace property
        self.namespace_properties_client.delete_namespace_property(
            namespace['namespace'], resource_name)
        # List namespace properties and validate deletion
        namespace_property = [
            namespace_property['title'] for namespace_property in
            self.namespace_properties_client.list_namespace_properties(
                namespace['namespace'])['properties']]
        self.assertNotIn(update_property_title, namespace_property)



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from oslo_log import log as logging
from testtools import matchers

from tempest.api.volume import base
from tempest.common import identity
from tempest.common import tempest_fixtures as fixtures
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

LOG = logging.getLogger(__name__)
QUOTA_KEYS = ['gigabytes', 'snapshots', 'volumes', 'backups',
              'backup_gigabytes', 'per_volume_gigabytes']


class VolumeQuotaClassesTest(base.BaseVolumeAdminTest):
    """Test volume quota classes"""

    def setUp(self):
        # Note(jeremy.zhang): All test cases in this class need to externally
        # lock on doing anything with default quota values.
        self.useFixture(fixtures.LockFixture('volume_quotas'))
        super(VolumeQuotaClassesTest, self).setUp()

    def _restore_default_quotas(self, original_defaults):
        LOG.debug("Restoring volume quota class defaults")
        self.admin_quota_classes_client.update_quota_class_set(
            'default', **original_defaults)

    @decorators.idempotent_id('abb9198e-67d0-4b09-859f-4f4a1418f176')
    def test_show_default_quota(self):
        """Test showing default volume quota class set"""
        # response body is validated by schema
        default_quotas = self.admin_quota_classes_client.show_quota_class_set(
            'default')['quota_class_set']
        self.assertEqual('default', default_quotas.pop('id'))

    @decorators.idempotent_id('a7644c63-2669-467a-b00e-452dd5c5397b')
    def test_update_default_quota(self):
        """Test updating default volume quota class set

        Check current project and new project's default quota are updated
        to the provided one.
        """
        LOG.debug("Get the current default quota class values")
        body = self.admin_quota_classes_client.show_quota_class_set(
            'default')['quota_class_set']

        # Note(jeremyZ) Only include specified quota keys to avoid the conflict
        # that other tests may create/delete volume types or update volume
        # type's default quotas in concurrency running.
        update_kwargs = {key: body[key] for key in body if key in QUOTA_KEYS}

        # Restore the defaults when the test is done.
        self.addCleanup(self._restore_default_quotas, update_kwargs.copy())

        # Note(jeremyZ) Increment some of the values for updating the default
        # quota class. For safety, only items with value >= 0 will be updated,
        # and items with value < 0 (-1 means unlimited) will be ignored.
        for quota, default in update_kwargs.items():
            if default >= 0:
                update_kwargs[quota] = default + 1

        # Create a volume type for updating default quotas class.
        volume_type_name = self.create_volume_type()['name']
        for key in ['volumes', 'snapshots', 'gigabytes']:
            update_kwargs['%s_%s' % (key, volume_type_name)] = \
                random.randint(1, 10)

        LOG.debug("Update limits for the default quota class set")
        update_body = self.admin_quota_classes_client.update_quota_class_set(
            'default', **update_kwargs)['quota_class_set']
        self.assertThat(update_body.items(),
                        matchers.ContainsAll(update_kwargs.items()))

        # Verify current project's default quotas.
        default_quotas = self.admin_quotas_client.show_default_quota_set(
            self.os_admin.credentials.tenant_id)['quota_set']
        self.assertThat(default_quotas.items(),
                        matchers.ContainsAll(update_kwargs.items()))

        # Verify a new project's default quotas.
        project_name = data_utils.rand_name('quota_class_tenant')
        description = data_utils.rand_name('desc_')
        project_id = identity.identity_utils(self.os_admin).create_project(
            name=project_name, description=description)['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        project_id)
        default_quotas = self.admin_quotas_client.show_default_quota_set(
            project_id)['quota_set']
        self.assertThat(default_quotas.items(),
                        matchers.ContainsAll(update_kwargs.items()))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from tempest.api.compute import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class FlavorsAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Flavors API Create and Delete that require admin privileges"""

    @classmethod
    def skip_checks(cls):
        super(FlavorsAdminTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('OS-FLV-EXT-DATA', 'compute'):
            msg = "OS-FLV-EXT-DATA extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(FlavorsAdminTestJSON, cls).resource_setup()

        cls.flavor_name_prefix = 'test_flavor_'
        cls.ram = 512
        cls.vcpus = 1
        cls.disk = 10
        cls.ephemeral = 10
        cls.swap = 1024
        cls.rxtx = 2

    @decorators.idempotent_id('8b4330e1-12c4-4554-9390-e6639971f086')
    def test_create_flavor_with_int_id(self):
        """Test creating flavor with id of type integer"""
        flavor_id = data_utils.rand_int_id(start=1000)
        new_flavor_id = self.create_flavor(ram=self.ram,
                                           vcpus=self.vcpus,
                                           disk=self.disk,
                                           id=flavor_id)['id']
        self.assertEqual(new_flavor_id, str(flavor_id))

    @decorators.idempotent_id('94c9bb4e-2c2a-4f3c-bb1f-5f0daf918e6d')
    def test_create_flavor_with_uuid_id(self):
        """Test creating flavor with id of type uuid"""
        flavor_id = data_utils.rand_uuid()
        new_flavor_id = self.create_flavor(ram=self.ram,
                                           vcpus=self.vcpus,
                                           disk=self.disk,
                                           id=flavor_id)['id']
        self.assertEqual(new_flavor_id, flavor_id)

    @decorators.idempotent_id('f83fe669-6758-448a-a85e-32d351f36fe0')
    def test_create_flavor_with_none_id(self):
        """Test creating flavor without id specified

        If nova receives a request with None as flavor_id,
        nova generates flavor_id of uuid.
        """
        flavor_id = None
        new_flavor_id = self.create_flavor(ram=self.ram,
                                           vcpus=self.vcpus,
                                           disk=self.disk,
                                           id=flavor_id)['id']
        self.assertEqual(new_flavor_id, str(uuid.UUID(new_flavor_id)))

    @decorators.idempotent_id('8261d7b0-be58-43ec-a2e5-300573c3f6c5')
    def test_create_flavor_verify_entry_in_list_details(self):
        """Create a flavor and ensure its details are listed

        This operation requires the user to have 'admin' role
        """
        flavor_name = data_utils.rand_name(self.flavor_name_prefix)

        # Create the flavor
        self.create_flavor(name=flavor_name,
                           ram=self.ram, vcpus=self.vcpus,
                           disk=self.disk,
                           ephemeral=self.ephemeral,
                           swap=self.swap,
                           rxtx_factor=self.rxtx)

        # Check if flavor is present in list
        flavors_list = self.admin_flavors_client.list_flavors(
            detail=True)['flavors']
        self.assertIn(flavor_name, [f['name'] for f in flavors_list])

    @decorators.idempotent_id('63dc64e6-2e79-4fdf-868f-85500d308d66')
    def test_create_list_flavor_without_extra_data(self):
        """Create a flavor and ensure it is listed

        This operation requires the user to have 'admin' role
        """
        def verify_flavor_response_extension(flavor):
            # check some extensions for the flavor create/show/detail response
            if self.is_requested_microversion_compatible('2.74'):
                self.assertEqual(flavor['swap'], '')
            else:
                self.assertEqual(flavor['swap'], 0)
            self.assertEqual(int(flavor['rxtx_factor']), 1)
            self.assertEqual(flavor['OS-FLV-EXT-DATA:ephemeral'], 0)
            self.assertEqual(flavor['os-flavor-access:is_public'], True)

        flavor_name = data_utils.rand_name(self.flavor_name_prefix)
        new_flavor_id = data_utils.rand_int_id(start=1000)

        # Create the flavor
        flavor = self.create_flavor(name=flavor_name,
                                    ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk,
                                    id=new_flavor_id)
        self.assertEqual(flavor['name'], flavor_name)
        self.assertEqual(flavor['ram'], self.ram)
        self.assertEqual(flavor['vcpus'], self.vcpus)
        self.assertEqual(flavor['disk'], self.disk)
        self.assertEqual(int(flavor['id']), new_flavor_id)
        verify_flavor_response_extension(flavor)

        # Verify flavor is retrieved
        flavor = self.admin_flavors_client.show_flavor(new_flavor_id)['flavor']
        self.assertEqual(flavor['name'], flavor_name)
        verify_flavor_response_extension(flavor)

        # Check if flavor is present in list
        flavors_list = [
            f for f in self.flavors_client.list_flavors(detail=True)['flavors']
            if f['name'] == flavor_name
        ]
        self.assertNotEmpty(flavors_list)
        verify_flavor_response_extension(flavors_list[0])

    @decorators.idempotent_id('be6cc18c-7c5d-48c0-ac16-17eaf03c54eb')
    def test_list_non_public_flavor(self):
        """Create a flavor with os-flavor-access:is_public false.

        The flavor should not be present in list_details as the
        tenant is not automatically added access list.
        This operation requires the user to have 'admin' role
        """
        flavor_name = data_utils.rand_name(self.flavor_name_prefix)

        # Create the flavor
        self.create_flavor(name=flavor_name,
                           ram=self.ram, vcpus=self.vcpus,
                           disk=self.disk,
                           is_public="False")
        # Verify flavor is not retrieved
        flavors_list = self.admin_flavors_client.list_flavors(
            detail=True)['flavors']
        self.assertNotIn(flavor_name, [f['name'] for f in flavors_list])

        # Verify flavor is not retrieved with other user
        flavors_list = self.flavors_client.list_flavors(detail=True)['flavors']
        self.assertNotIn(flavor_name, [f['name'] for f in flavors_list])

    @decorators.idempotent_id('bcc418ef-799b-47cc-baa1-ce01368b8987')
    def test_create_server_with_non_public_flavor(self):
        """Create a flavor with os-flavor-access:is_public false"""
        flavor = self.create_flavor(ram=self.ram, vcpus=self.vcpus,
                                    disk=self.disk,
                                    is_public="False")

        # Verify flavor is not used by other user
        self.assertRaises(lib_exc.BadRequest,
                          self.os_primary.servers_client.create_server,
                          name='test', imageRef=self.image_ref,
                          flavorRef=flavor['id'])

    @decorators.idempotent_id('b345b196-bfbd-4231-8ac1-6d7fe15ff3a3')
    def test_list_public_flavor_with_other_user(self):
        """Create a Flavor with public access.

        Try to List/Get flavor with another user
        """
        flavor_name = data_utils.rand_name(self.flavor_name_prefix)

        # Create the flavor
        self.create_flavor(name=flavor_name,
                           ram=self.ram, vcpus=self.vcpus,
                           disk=self.disk,
                           is_public="True")
        # Verify flavor is retrieved with new user
        flavors_list = self.flavors_client.list_flavors(detail=True)['flavors']
        self.assertIn(flavor_name, [f['name'] for f in flavors_list])

    @decorators.idempotent_id('fb9cbde6-3a0e-41f2-a983-bdb0a823c44e')
    def test_is_public_string_variations(self):
        """Test creating public and non public flavors"""
        flavor_name_not_public = data_utils.rand_name(self.flavor_name_prefix)
        flavor_name_public = data_utils.rand_name(self.flavor_name_prefix)

        # Create a non public flavor
        self.create_flavor(name=flavor_name_not_public,
                           ram=self.ram, vcpus=self.vcpus,
                           disk=self.disk,
                           is_public="False")

        # Create a public flavor
        self.create_flavor(name=flavor_name_public,
                           ram=self.ram, vcpus=self.vcpus,
                           disk=self.disk,
                           is_public="True")

        def _test_string_variations(variations, flavor_name):
            for string in variations:
                params = {'is_public': string}
                flavors = (self.admin_flavors_client.list_flavors(detail=True,
                                                                  **params)
                           ['flavors'])
                self.assertIn(flavor_name, [f['name'] for f in flavors])

        _test_string_variations(['f', 'false', 'no', '0'],
                                flavor_name_not_public)

        _test_string_variations(['t', 'true', 'yes', '1'],
                                flavor_name_public)

    @decorators.idempotent_id('3b541a2e-2ac2-4b42-8b8d-ba6e22fcd4da')
    def test_create_flavor_using_string_ram(self):
        """Test creating flavor with ram of type string"""
        new_flavor_id = data_utils.rand_int_id(start=1000)

        ram = "1024"
        flavor = self.create_flavor(ram=ram, vcpus=self.vcpus,
                                    disk=self.disk,
                                    id=new_flavor_id)
        self.assertEqual(flavor['ram'], int(ram))
        self.assertEqual(int(flavor['id']), new_flavor_id)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import migrations_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestMigrationsClient(base.BaseServiceTest):
    FAKE_MIGRATION_INFO = {"migrations": [{
        "created_at": "2012-10-29T13:42:02",
        "dest_compute": "compute2",
        "dest_host": "1.2.3.4",
        "dest_node": "node2",
        "id": 1234,
        "instance_uuid": "e9e4fdd7-f956-44ff-bfeb-d654a96ab3a2",
        "new_instance_type_id": 2,
        "old_instance_type_id": 1,
        "source_compute": "compute1",
        "source_node": "node1",
        "status": "finished",
        "updated_at": "2012-10-29T13:42:02"}]}

    def setUp(self):
        super(TestMigrationsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.mg_client_obj = migrations_client.MigrationsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_migrations(self, bytes_body=False):
        self.check_service_client_function(
            self.mg_client_obj.list_migrations,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_MIGRATION_INFO,
            bytes_body)

    def test_list_migration_with_str_body(self):
        self._test_list_migrations()

    def test_list_migration_with_bytes_body(self):
        self._test_list_migrations(True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.network import base
from tempest.common import identity
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class RoutersAdminTest(base.BaseAdminNetworkTest):
    """Test routers operation supported by admin"""

    # NOTE(salv-orlando): This class inherits from BaseAdminNetworkTest
    # as some router operations, such as enabling or disabling SNAT
    # require admin credentials by default

    def _cleanup_router(self, router):
        self.delete_router(router)

    def _create_router(self, name=None, admin_state_up=False,
                       external_network_id=None, enable_snat=None):
        # associate a cleanup with created routers to avoid quota limits
        router = self.create_router(name, admin_state_up,
                                    external_network_id, enable_snat)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._cleanup_router, router)
        return router

    @classmethod
    def skip_checks(cls):
        super(RoutersAdminTest, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)

    @decorators.idempotent_id('e54dd3a3-4352-4921-b09d-44369ae17397')
    def test_create_router_setting_project_id(self):
        """Test creating router from admin user setting project_id."""
        project = data_utils.rand_name('test_tenant_')
        description = data_utils.rand_name('desc_')
        project = identity.identity_utils(self.os_admin).create_project(
            name=project, description=description)
        project_id = project['id']
        self.addCleanup(identity.identity_utils(self.os_admin).delete_project,
                        project_id)

        name = data_utils.rand_name('router-')
        create_body = self.admin_routers_client.create_router(
            name=name, project_id=project_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_routers_client.delete_router,
                        create_body['router']['id'])
        self.assertEqual(project_id, create_body['router']['project_id'])

    @decorators.idempotent_id('847257cc-6afd-4154-b8fb-af49f5670ce8')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_create_router_with_default_snat_value(self):
        """Create a router with default snat rule"""
        router = self._create_router(
            external_network_id=CONF.network.public_network_id)
        self._verify_router_gateway(
            router['id'], {'network_id': CONF.network.public_network_id,
                           'enable_snat': True})

    @decorators.idempotent_id('ea74068d-09e9-4fd7-8995-9b6a1ace920f')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_create_router_with_snat_explicit(self):
        """Test creating router with specified enable_snat value"""
        name = data_utils.rand_name('snat-router')
        # Create a router enabling snat attributes
        enable_snat_states = [False, True]
        for enable_snat in enable_snat_states:
            external_gateway_info = {
                'network_id': CONF.network.public_network_id,
                'enable_snat': enable_snat}
            create_body = self.admin_routers_client.create_router(
                name=name, external_gateway_info=external_gateway_info)
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.admin_routers_client.delete_router,
                            create_body['router']['id'])
            # Verify snat attributes after router creation
            self._verify_router_gateway(create_body['router']['id'],
                                        exp_ext_gw_info=external_gateway_info)

    def _verify_router_gateway(self, router_id, exp_ext_gw_info=None):
        show_body = self.admin_routers_client.show_router(router_id)
        actual_ext_gw_info = show_body['router']['external_gateway_info']
        if exp_ext_gw_info is None:
            self.assertIsNone(actual_ext_gw_info)
            return
        # Verify only keys passed in exp_ext_gw_info
        for k, v in exp_ext_gw_info.items():
            self.assertEqual(v, actual_ext_gw_info[k])

    def _verify_gateway_port(self, router_id):
        list_body = self.admin_ports_client.list_ports(
            network_id=CONF.network.public_network_id,
            device_id=router_id,
            device_owner="network:router_gateway")
        self.assertEqual(len(list_body['ports']), 1)
        gw_port = list_body['ports'][0]
        fixed_ips = gw_port['fixed_ips']
        self.assertNotEmpty(fixed_ips)
        # Assert that all of the IPs from the router gateway port
        # are allocated from a valid public subnet.
        public_net_body = self.admin_networks_client.show_network(
            CONF.network.public_network_id)
        public_subnet_ids = public_net_body['network']['subnets']
        for fixed_ip in fixed_ips:
            subnet_id = fixed_ip['subnet_id']
            self.assertIn(subnet_id, public_subnet_ids)

    @decorators.idempotent_id('6cc285d8-46bf-4f36-9b1a-783e3008ba79')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_update_router_set_gateway(self):
        """Test updating router's gateway info"""
        router = self._create_router()
        self.routers_client.update_router(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id})
        # Verify operation - router
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('b386c111-3b21-466d-880c-5e72b01e1a33')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_update_router_set_gateway_with_snat_explicit(self):
        """Test setting router's gateway with snat enabled"""
        router = self._create_router()
        self.admin_routers_client.update_router(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': True})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': True})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('96536bc7-8262-4fb2-9967-5c46940fa279')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_update_router_set_gateway_without_snat(self):
        """Test setting router's gateway with snat not enabled"""
        router = self._create_router()
        self.admin_routers_client.update_router(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': False})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': False})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('ad81b7ee-4f81-407b-a19c-17e623f763e8')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_update_router_unset_gateway(self):
        """Test unsetting router's gateway"""
        router = self._create_router(
            external_network_id=CONF.network.public_network_id)
        self.routers_client.update_router(router['id'],
                                          external_gateway_info={})
        self._verify_router_gateway(router['id'])
        # No gateway port expected
        list_body = self.admin_ports_client.list_ports(
            network_id=CONF.network.public_network_id,
            device_id=router['id'])
        self.assertFalse(list_body['ports'])

    @decorators.idempotent_id('f2faf994-97f4-410b-a831-9bc977b64374')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_update_router_reset_gateway_without_snat(self):
        """Test updating router's gateway to be with snat not enabled"""
        router = self._create_router(
            external_network_id=CONF.network.public_network_id)
        self.admin_routers_client.update_router(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': False})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': False})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('cbe42f84-04c2-11e7-8adb-fa163e4fa634')
    @utils.requires_ext(extension='ext-gw-mode', service='network')
    def test_create_router_set_gateway_with_fixed_ip(self):
        """Test creating router setting gateway with fixed ip"""
        # At first create an external network and then use that
        # to create address and delete
        network_name = data_utils.rand_name(self.__class__.__name__)
        network_1 = self.admin_networks_client.create_network(
            name=network_name, **{'router:external': True})['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_networks_client.delete_network,
                        network_1['id'])
        subnet = self.create_subnet(
            network_1, client=self.admin_subnets_client, enable_dhcp=False)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_subnets_client.delete_subnet, subnet['id'])
        port = self.admin_ports_client.create_port(
            name=data_utils.rand_name(self.__class__.__name__),
            network_id=network_1['id'])['port']
        self.admin_ports_client.delete_port(port_id=port['id'])
        fixed_ip = {
            'subnet_id': port['fixed_ips'][0]['subnet_id'],
            'ip_address': port['fixed_ips'][0]['ip_address']
        }
        external_gateway_info = {
            'network_id': network_1['id'],
            'external_fixed_ips': [fixed_ip]
        }
        # Create a router and set gateway to fixed_ip
        router = self.admin_routers_client.create_router(
            external_gateway_info=external_gateway_info)['router']
        self.admin_routers_client.delete_router(router['id'])
        # Examine router's gateway is equal to fixed_ip
        self.assertEqual(router['external_gateway_info'][
                         'external_fixed_ips'][0]['ip_address'],
                         fixed_ip['ip_address'])


class RoutersIpV6AdminTest(RoutersAdminTest):
    _ip_version = 6



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v3 import inherited_roles_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestInheritedRolesClient(base.BaseServiceTest):
    FAKE_LIST_INHERITED_ROLES = {
        "roles": [
            {
                "id": "1",
                "name": "test",
                "links": "example.com"
            },
            {
                "id": "2",
                "name": "test2",
                "links": "example.com"
            }
        ]
    }

    def setUp(self):
        super(TestInheritedRolesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = inherited_roles_client.InheritedRolesClient(
            fake_auth, 'identity', 'regionOne')

    def _test_create_inherited_role_on_domains_user(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_inherited_role_on_domains_user,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def _test_list_inherited_project_role_for_user_on_domain(
        self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_inherited_project_role_for_user_on_domain,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_INHERITED_ROLES,
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123")

    def _test_create_inherited_role_on_domains_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_inherited_role_on_domains_group,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def _test_list_inherited_project_role_for_group_on_domain(
        self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_inherited_project_role_for_group_on_domain,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_INHERITED_ROLES,
            bytes_body,
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123")

    def _test_create_inherited_role_on_projects_user(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_inherited_role_on_projects_user,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def _test_create_inherited_role_on_projects_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_inherited_role_on_projects_group,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_create_inherited_role_on_domains_user_with_str_body(self):
        self._test_create_inherited_role_on_domains_user()

    def test_create_inherited_role_on_domains_user_with_bytes_body(self):
        self._test_create_inherited_role_on_domains_user(bytes_body=True)

    def test_create_inherited_role_on_domains_group_with_str_body(self):
        self._test_create_inherited_role_on_domains_group()

    def test_create_inherited_role_on_domains_group_with_bytes_body(self):
        self._test_create_inherited_role_on_domains_group(bytes_body=True)

    def test_create_inherited_role_on_projects_user_with_str_body(self):
        self._test_create_inherited_role_on_projects_user()

    def test_create_inherited_role_on_projects_group_with_bytes_body(self):
        self._test_create_inherited_role_on_projects_group(bytes_body=True)

    def test_list_inherited_project_role_for_user_on_domain_with_str_body(
        self):
        self._test_list_inherited_project_role_for_user_on_domain()

    def test_list_inherited_project_role_for_user_on_domain_with_bytes_body(
        self):
        self._test_list_inherited_project_role_for_user_on_domain(
            bytes_body=True)

    def test_list_inherited_project_role_for_group_on_domain_with_str_body(
        self):
        self._test_list_inherited_project_role_for_group_on_domain()

    def test_list_inherited_project_role_for_group_on_domain_with_bytes_body(
        self):
        self._test_list_inherited_project_role_for_group_on_domain(
            bytes_body=True)

    def test_delete_inherited_role_from_user_on_domain(self):
        self.check_service_client_function(
            self.client.delete_inherited_role_from_user_on_domain,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_check_user_inherited_project_role_on_domain(self):
        self.check_service_client_function(
            self.client.check_user_inherited_project_role_on_domain,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_delete_inherited_role_from_group_on_domain(self):
        self.check_service_client_function(
            self.client.delete_inherited_role_from_group_on_domain,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_check_group_inherited_project_role_on_domain(self):
        self.check_service_client_function(
            self.client.check_group_inherited_project_role_on_domain,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            domain_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_delete_inherited_role_from_user_on_project(self):
        self.check_service_client_function(
            self.client.delete_inherited_role_from_user_on_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_check_user_has_flag_on_inherited_to_project(self):
        self.check_service_client_function(
            self.client.check_user_has_flag_on_inherited_to_project,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)

    def test_delete_inherited_role_from_group_on_project(self):
        self.check_service_client_function(
            self.client.delete_inherited_role_from_group_on_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)

    def test_check_group_has_flag_on_inherited_to_project(self):
        self.check_service_client_function(
            self.client.check_group_has_flag_on_inherited_to_project,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            project_id="b344506af7644f6794d9cb316600b020",
            group_id="123",
            role_id="1234",
            status=204)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common import api_version_request
from tempest.lib.common import api_version_utils
from tempest.lib.common import rest_client
from tempest.lib import exceptions

COMPUTE_MICROVERSION = None


class BaseComputeClient(rest_client.RestClient):
    """Base compute service clients class to support microversion.

    This class adds microversion to API request header if that is set
    and provides interface to select appropriate JSON schema file for
    response validation.

    :param auth_provider: An auth provider object used to wrap requests in
                          auth
    :param str service: The service name to use for the catalog lookup
    :param str region: The region to use for the catalog lookup
    :param kwargs: kwargs required by rest_client.RestClient
    """

    api_microversion_header_name = 'X-OpenStack-Nova-API-Version'

    def get_headers(self):
        headers = super(BaseComputeClient, self).get_headers()
        if COMPUTE_MICROVERSION:
            headers[self.api_microversion_header_name] = COMPUTE_MICROVERSION
        return headers

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):
        resp, resp_body = super(BaseComputeClient, self).request(
            method, url, extra_headers, headers, body, chunked)
        if (COMPUTE_MICROVERSION and
            COMPUTE_MICROVERSION != api_version_utils.LATEST_MICROVERSION):
            api_version_utils.assert_version_header_matches_request(
                self.api_microversion_header_name,
                COMPUTE_MICROVERSION,
                resp)
        return resp, resp_body

    def get_schema(self, schema_versions_info):
        """Get JSON schema

        This method provides the matching schema for requested
        microversion.

        :param schema_versions_info: List of dict which provides schema
                                     information with range of valid versions.

        Example::

         schema_versions_info = [
             {'min': None, 'max': '2.1', 'schema': schemav21},
             {'min': '2.2', 'max': '2.9', 'schema': schemav22},
             {'min': '2.10', 'max': None, 'schema': schemav210}]
        """
        schema = None
        version = api_version_request.APIVersionRequest(COMPUTE_MICROVERSION)
        for items in schema_versions_info:
            min_version = api_version_request.APIVersionRequest(items['min'])
            max_version = api_version_request.APIVersionRequest(items['max'])
            # This is case where COMPUTE_MICROVERSION is None, which means
            # request without microversion So select base v2.1 schema.
            if version.is_null() and items['min'] is None:
                schema = items['schema']
                break
            # else select appropriate schema as per COMPUTE_MICROVERSION
            elif version.matches(min_version, max_version):
                schema = items['schema']
                break
        if schema is None:
            raise exceptions.JSONSchemaNotFound(
                version=version.get_string(),
                schema_versions_info=schema_versions_info)
        return schema



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import custom_matchers
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager

CONF = config.CONF


class TestMinimumBasicScenario(manager.ScenarioTest):

    """This is a basic minimum scenario test.

    This test below:
    * across the multiple components
    * as a regular user
    * with and without optional parameters
    * check command outputs

    Steps:
    1. Create image
    2. Create keypair
    3. Boot instance with keypair and get list of instances
    4. Create volume and show list of volumes
    5. Attach volume to instance and getlist of volumes
    6. Add IP to instance
    7. Create and add security group to instance
    8. Check SSH connection to instance
    9. Reboot instance
    10. Check SSH connection to instance after reboot

    """

    def nova_show(self, server):
        got_server = (self.servers_client.show_server(server['id'])
                      ['server'])
        excluded_keys = ['OS-EXT-AZ:availability_zone']
        # Exclude these keys because of LP:#1486475
        excluded_keys.extend(['OS-EXT-STS:power_state', 'updated'])
        self.assertThat(
            server, custom_matchers.MatchesDictExceptForKeys(
                got_server, excluded_keys=excluded_keys))

    def cinder_show(self, volume):
        got_volume = self.volumes_client.show_volume(volume['id'])['volume']
        # Exclude updated_at because of bug 1838202.
        excluded_keys = ['updated_at']
        self.assertThat(
            volume, custom_matchers.MatchesDictExceptForKeys(
                got_volume, excluded_keys=excluded_keys))

    def nova_reboot(self, server):
        self.servers_client.reboot_server(server['id'], type='SOFT')
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

    def check_disks(self):
        # NOTE(andreaf) The device name may be different on different guest OS
        disks = self.linux_client.get_disks()
        self.assertEqual(1, disks.count(CONF.compute.volume_device_name))

    def create_and_add_security_group_to_server(self, server):
        secgroup = self.create_security_group()
        self.servers_client.add_security_group(server['id'],
                                               name=secgroup['name'])
        self.addCleanup(self.servers_client.remove_security_group,
                        server['id'], name=secgroup['name'])

        def wait_for_secgroup_add():
            body = (self.servers_client.show_server(server['id'])
                    ['server'])
            return {'name': secgroup['name']} in body['security_groups']

        if not test_utils.call_until_true(wait_for_secgroup_add,
                                          CONF.compute.build_timeout,
                                          CONF.compute.build_interval):
            msg = ('Timed out waiting for adding security group %s to server '
                   '%s' % (secgroup['id'], server['id']))
            raise exceptions.TimeoutException(msg)

    def _get_floating_ip_in_server_addresses(self, floating_ip, server):
        for addresses in server['addresses'].values():
            for address in addresses:
                if (address['OS-EXT-IPS:type'] == 'floating' and
                        address['addr'] == floating_ip['floating_ip_address']):
                    return address

    @decorators.idempotent_id('bdbb5441-9204-419d-a225-b4fdbfb1a1a8')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_minimum_basic_scenario(self):
        image = self.image_create()
        keypair = self.create_keypair()

        server = self.create_server(image_id=image, key_name=keypair['name'])
        servers = self.servers_client.list_servers()['servers']
        self.assertIn(server['id'], [x['id'] for x in servers])

        self.nova_show(server)

        volume = self.create_volume()
        volumes = self.volumes_client.list_volumes()['volumes']
        self.assertIn(volume['id'], [x['id'] for x in volumes])

        self.cinder_show(volume)

        volume = self.nova_volume_attach(server, volume)
        self.addCleanup(self.nova_volume_detach, server, volume)
        self.cinder_show(volume)

        floating_ip = None
        server = self.servers_client.show_server(server['id'])['server']
        if (CONF.network_feature_enabled.floating_ips and
            CONF.network.floating_network_name):
            fip = self.create_floating_ip(server)
            floating_ip = self.associate_floating_ip(
                fip, server)
            # fetch the server again to make sure the addresses were refreshed
            # after associating the floating IP
            server = self.servers_client.show_server(server['id'])['server']
            address = self._get_floating_ip_in_server_addresses(
                floating_ip, server)
            self.assertIsNotNone(
                address,
                "Failed to find floating IP '%s' in server addresses: %s" %
                (floating_ip['floating_ip_address'], server['addresses']))
            ssh_ip = floating_ip['floating_ip_address']
        else:
            ssh_ip = self.get_server_ip(server)

        self.create_and_add_security_group_to_server(server)

        # check that we can SSH to the server before reboot
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        self.nova_reboot(server)

        # check that we can SSH to the server after reboot
        # (both connections are part of the scenario)
        self.linux_client = self.get_remote_client(
            ssh_ip, private_key=keypair['private_key'],
            server=server)

        self.check_disks()

        if floating_ip:
            # delete the floating IP, this should refresh the server addresses
            self.disassociate_floating_ip(floating_ip)

            def is_floating_ip_detached_from_server():
                server_info = self.servers_client.show_server(
                    server['id'])['server']
                address = self._get_floating_ip_in_server_addresses(
                    floating_ip, server_info)
                return (not address)

            if not test_utils.call_until_true(
                is_floating_ip_detached_from_server,
                CONF.compute.build_timeout,
                CONF.compute.build_interval):
                msg = ("Floating IP '%s' should not be in server addresses: %s"
                       % (floating_ip['floating_ip_address'],
                          server['addresses']))
                raise exceptions.TimeoutException(msg)
